TupleExtendError

Struct TupleExtendError 

pub struct TupleExtendError<E, P, I> {
    pub error: E,
    pub pending: Option<P>,
    pub remaining: I,
}
An error that occurs when extending a tuple of collections fails.

This error preserves the error from the side that failed, any pending item from the other side, and the remaining iterator for error recovery.

Note this type is read-only. The fields are accessible via a hidden Deref implementation into a hidden TupleExtendErrorData type, with identical fields. If necessary, you can consume an instance of this type via TupleExtendError::into_data to get owned data.

Type Parameters

  • E: The error type from the failing collection.
  • P: The type of the pending item from the other collection.
  • I: The type of the remaining iterator.

Fields

error: E

The error that occurred during extension.

pending: Option<P>

The pending item from the other side, if any.

remaining: I

The remaining iterator after the error occurred

Implementations

Source

impl<E, P, I> TupleExtendError<E, P, I>

pub fn into_data(self) -> TupleExtendErrorData<E, P, I>

Consumes the error, returning the data.