pub struct CapacityError<T> {
pub capacity: SizeHint,
pub kind: CapacityErrorKind<T>,
}Expand description
An error indicating that a collection or extension operation failed because
of a conflict between the collection’s capacity and the number of items
produced by an Iterator.
This error represents three failure modes, identified by CapacityError::kind.
-
CapacityErrorKind::Bounds— the iterator’s reported size bounds are incompatible with the required capacity, making the operation impossible to satisfy even before iteration begins. -
CapacityErrorKind::Underflow— the iterator produced fewer items than the minimum required by the capacity. -
CapacityErrorKind::Overflow— the iterator produced more items than the maximum allowed by the capacity. The item that caused the overflow is preserved, and may be recovered viaErrorItemProvider. The number of overflowing items is implied to be at least one, but may be more.
Type Parameters
T: The type of the item in the collection.
Fields
capacity: SizeHintThe capacity constraint that was violated.
kind: CapacityErrorKind<T>The specific kind of capacity mismatch that occurred.
Implementations
Sourceimpl<T> CapacityError<T>
impl<T> CapacityError<T>
Sourcepub const fn bounds(capacity: SizeHint, hint: SizeHint) -> Self
pub const fn bounds(capacity: SizeHint, hint: SizeHint) -> Self
Creates a new Bounds CapacityError
indicating that hint bounds are incompatible with capacity.
Arguments
capacity- the capacity of the collectionhint- the size hint of the iterator
Panics
Panics if capacity and hint overlap.
Examples
let err = CapacityError::<i32>::bounds(SizeHint::exact(5), SizeHint::exact(2));
assert_eq!(err.capacity, SizeHint::exact(5));
assert_eq!(err.kind, CapacityErrorKind::Bounds { hint: SizeHint::exact(2) });Sourcepub fn ensure_fits<I: Iterator<Item = T>>(
iter: &I,
capacity: SizeHint,
) -> Result<(), Self>
pub fn ensure_fits<I: Iterator<Item = T>>( iter: &I, capacity: SizeHint, ) -> Result<(), Self>
Ensures that iter’s size_hint overlaps
capacity.
Arguments
iter- theIteratorto check.capacity- the capacity of the collection
Errors
Returns a Bounds CapacityError if
iter’s size_hint is disjoint
with cap.
Panics
Panics if iter’s size_hint is invalid.
Examples
CapacityError::<i32>::ensure_fits(&(1..=5), SizeHint::at_most(5)).expect("Should fit");
let err = CapacityError::<i32>::ensure_fits(&(1..=5), SizeHint::at_most(4))
.expect_err("Should fit");
assert_eq!(err.capacity, SizeHint::at_most(4));
assert_eq!(err.kind, CapacityErrorKind::Bounds { hint: SizeHint::exact(5) });Sourcepub const fn overflow(capacity: SizeHint, overflow: T) -> Self
pub const fn overflow(capacity: SizeHint, overflow: T) -> Self
Creates a new CapacityError indicating that the collection
overflowed capacity.
Arguments
capacity- The capacity that overflowed.overflow- The item that overflowedcapacity.
Panics
Panics if capacity is not bounded.
Examples
let err = CapacityError::overflow(SizeHint::exact(5), 2);
assert_eq!(err.capacity, SizeHint::exact(5));
assert_eq!(err.kind, CapacityErrorKind::Overflow { overflow: 2 });Sourcepub const fn underflow(capacity: SizeHint, count: usize) -> Self
pub const fn underflow(capacity: SizeHint, count: usize) -> Self
Creates a new CapacityError indicating that fewer items were produced
than the minimum required by capacity.
Arguments
capacity- The capacity that was underflowed.count- The number of items that were produced.
Panics
Panics if count is greater than or equal to capacity.lower().
Examples
let err = CapacityError::<i32>::underflow(SizeHint::exact(5), 2);
assert_eq!(err.capacity, SizeHint::exact(5));
assert_eq!(err.kind, CapacityErrorKind::Underflow { count: 2 });Sourcepub const fn underflow_of<C: FixedCap>(count: usize) -> Self
pub const fn underflow_of<C: FixedCap>(count: usize) -> Self
Creates a new CapacityError indicating that fewer items were produced
than the minimum required by C’s FixedCap::CAP.
Arguments
count- The number of items that were produced.
Type Parameters
C- A collection with a fixed capaciy.
Examples
let err = CapacityError::<i32>::underflow_of::<[i32; 5]>(2);
assert_eq!(err.capacity, SizeHint::exact(5));
assert_eq!(err.kind, CapacityErrorKind::Underflow { count: 2 });Trait Implementations
Sourceimpl<T: Debug> Debug for CapacityError<T>
impl<T: Debug> Debug for CapacityError<T>
Sourceimpl<T> Display for CapacityError<T>where
CapacityErrorKind<T>: Display,
impl<T> Display for CapacityError<T>where
CapacityErrorKind<T>: Display,
Sourceimpl<T> Error for CapacityError<T>
impl<T> Error for CapacityError<T>
1.30.0 · Sourcefn source(&self) -> Option<&(dyn Error + 'static)>
fn source(&self) -> Option<&(dyn Error + 'static)>
1.0.0 · Sourcefn description(&self) -> &str
fn description(&self) -> &str
Sourceimpl<T> ErrorItemProvider for CapacityError<T>
impl<T> ErrorItemProvider for CapacityError<T>
Sourceimpl<T> From<CapacityError<T>> for CapacityError<T>
Available on crate feature arrayvec only.
impl<T> From<CapacityError<T>> for CapacityError<T>
arrayvec only.Sourcefn from(err: CapacityError<T>) -> Self
fn from(err: CapacityError<T>) -> Self
Sourceimpl<T: PartialEq> PartialEq for CapacityError<T>
impl<T: PartialEq> PartialEq for CapacityError<T>
impl<T: Eq> Eq for CapacityError<T>
impl<T> StructuralPartialEq for CapacityError<T>